@@ -12,7 +12,7 @@ from .i18n import _
 RUNNING_AS_ROOT = os.geteuid() == 0
 
 
-VERSION = '1.6.12-dev'
+VERSION = '1.6.13-dev'
 
 _USER_CACHE_HOME = os.environ.get(
     "XDG_CACHE_HOME",
@@ -188,6 +188,18 @@ CONFIG_SCHEMA: Dict[str, Any] = {
             'type': 'bool',
             'default': 'no'
         },
+        'AurSearchSorting': {
+            'type': 'str',
+            'default': 'hottest'
+        },
+        'DisplayLastUpdated': {
+            'type': 'bool',
+            'default': 'no'
+        },
+        'GroupByRepository': {
+            'type': 'bool',
+            'default': 'yes'
+        },
         'ReverseSearchSorting': {
             'type': 'bool',
             'default': 'no'
